#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
//#include "LaTeX Pictures"

Function KillAllGraphs()
    string fulllist = WinList("*", ";","WIN:1")
    string name, cmd
    variable i
   
    for(i=0; i<itemsinlist(fulllist); i +=1)
        name= stringfromlist(i, fulllist)
        sprintf  cmd, "Dowindow/K %s", name
        execute cmd    
    endfor
end

Function DebyeIntegrand(inX)//See Ashcroft & Mermin Chapter 23, particularly page 459 equation 23.26. This is also discussed in many other texts on solid state physics and statistical mechanics
    Variable inX ;
    Variable result ;
    result = inX^4 * exp(inX) / ( exp(inX) - 1 )^2;
    return result;

End


Function DebyeInterpolation(DebyeCoef,x) : FitFunc //x is temperature

    //CurveFitDialog/
    //CurveFitDialog/ Coefficients 2
    //CurveFitDialog/ DebyeCoef[0] = Theta
    //CurveFitDialog/ DebyeCoef[1] = Scale

    //Recall that Cv = 3 N kb fd( Theta / T ) = 3 n R fd( Theta / T ) . See Function DebyeIntegrand(inX) and comments
    //fd() is described by DebyeFunctionFactor in this code. fd(y) is 3/y^3 * Integrate1D( X^4 * exp(X) / ( exp(X) + 1 )^2, 0, y )
    //where y = Theta/T
    // Cv/n, where (n) is #mols**  is 3 R fd( Theta / T )
    // **mols of what? formula units or atoms? This is why there is a scale factor DebyeCoef[1]. Also, sometimes people divide by 3R due to the DuLong Petit limit
    WAVE DebyeCoef ;
    Variable x ;
    
    Variable Rgas = 8.3144621 ; //Gas constant in J/mol/K
    Variable DebyeFunctionFactor ; //The second parameter DebyeCoef_1 is held fixed. You need to be plotting C/molAtoms, but if (in example) you rather plot C/molFormulaUnit and have (m) atoms in formula unit, set and hold this to (1/m).
    Variable result ;
    
    DebyeFunctionFactor = 3 * Integrate1D( DebyeIntegrand, 1e-13, DebyeCoef[0]/x) / ( DebyeCoef[0] / x )^3 ;//Note that we can't start the integral at actual zero
    
    result = 3 * Rgas * DebyeFunctionFactor ;
    
    If( DebyeCoef[1] )
        result = result * DebyeCoef[1]
    EndIf
    
    return result ;

End

Function DebyeInterpolationWithLinearT(DebyeCoef,x) : FitFunc //x is temperature
    //CurveFitDialog/
    //CurveFitDialog/ Coefficients 4
    //CurveFitDialog/ DebyeCoef[0] = Theta
    //CurveFitDialog/ DebyeCoef[1] = GammaHT
    //CurveFitDialog/ DebyeCoef[2] = Scale
    //CurveFitDialog/ DebyeCoef[3] = ElecRatio
    
    //Recall that Cv = 3 N kb fd( Theta / T ) = 3 n R fd( Theta / T ) . See Function DebyeIntegrand(inX) and comments
    //fd() is described by DebyeFunctionFactor in this code. fd(y) is 3/y^3 * Integrate1D( X^4 * exp(X) / ( exp(X) + 1 )^2, 0, y )
    //where y = Theta/T
    // Cv/n, where (n) is #mols**  is 3 R fd( Theta / T )
    // **mols of what? formula units or atoms? This is why there is a scale factor, DebyeCoef[2].
    WAVE DebyeCoef ;
    Variable x ;
    
    Variable Rgas = 8.3144621 ; //Gas constant in J/mol/K. Not used here since we normalize Cp by dividing by 3R
    Variable DebyeFunctionFactor ; 
    Variable result ;
    
    DebyeFunctionFactor = 3 * Integrate1D( DebyeIntegrand, 1e-13, DebyeCoef[0]/x) / ( DebyeCoef[0] / x )^3 ;//Note that we can't start the integral at actual zero
    
    result = 3 * Rgas * DebyeFunctionFactor + DebyeCoef[3] * DebyeCoef[1] * x ; //DebyeCoef[3] is set and held equal to (the number of electronic atoms in the unit cell you use)^(-1).
    
    If( DebyeCoef[2] )
        result = result * DebyeCoef[2] ;
    EndIf
    
    return result ;
End

Function Schottky(SchottkyCoef, x) : FitFunc //x is temperature
    //CurveFitDialog/
    //CurveFitDialog/ Coefficients 4
    //CurveFitDialog/ SchottkyCoef[0] = Delta
    //CurveFitDialog/ SchottkyCoef[1] = g0Perg1
    //CurveFitDialog/ SchottkyCoef[2] = ScaleFctr
    //CurveFitDialog/ SchottkyCoef[3] = ConstScale
    WAVE SchottkyCoef ;
    variable x ;
    
    variable Rgas ; Rgas = 8.3144621 ;
    variable returnValue; 
    returnValue = Rgas * SchottkyCoef[1] * ( SchottkyCoef[0] / x )^2 ;
    returnValue = returnValue * Exp( SchottkyCoef[0] / x ) / ( 1 + SchottkyCoef[1] * Exp( SchottkyCoef[0] / x ) )^2 ;
    
    If( SchottkyCoef[2] )//At the moment, the fit is in units of Cp/molAtom. However, you may not expect all atoms to contribute to the magnetic specific heat. There may only be (n) electronic atoms per unit cell and perhaps only (m) of those are in an oxidation state in which there is a(are) conduction electron(s)
        returnValue = returnValue * SchottkyCoef[2] ;
    EndIf
    If( SchottkyCoef[3] )//Sometimes you rescale your axis so that the units are not J/molK (example divide by DuLong Petit limit)
        returnValue = returnValue * SchottkyCoef[3] ;
    EndIf
    
    return returnValue ;
End

Function SchottkyPerT(SchottkyPerTCoef, x) : FitFunc //x is temperature
    //CurveFitDialog/
    //CurveFitDialog/ Coefficients 4
    //CurveFitDialog/ SchottkyPerTCoef[0] = Delta
    //CurveFitDialog/ SchottkyPerTCoef[1] = g0Perg1
    //CurveFitDialog/ SchottkyPerTCoef[2] = ScaleFctr
    //CurveFitDialog/ SchottkyPerTCoef[3] = ConstScale
    WAVE SchottkyPerTCoef ;
    variable x ;
    
    variable Rgas ; Rgas = 8.3144621 ;
    variable returnValue; 
    returnValue = Rgas * SchottkyPerTCoef[1] * ( SchottkyPerTCoef[0] / x )^2 ;
    returnValue = returnValue * Exp( SchottkyPerTCoef[0] / x ) / ( 1 + SchottkyPerTCoef[1] * Exp( SchottkyPerTCoef[0] / x ) )^2 ;
    returnValue = returnValue / x ;
    
    If( SchottkyPerTCoef[2] )//At the moment, the fit is in units of Cp/molAtom. However, you may not expect all atoms to contribute to the magnetic specific heat. There may only be (n) electronic atoms per unit cell and perhaps only (m) of those are in an oxidation state in which there is a(are) conduction electron(s)
        returnValue = returnValue * SchottkyPerTCoef[2] ;
    EndIf
    If( SchottkyPerTCoef[3] )//Sometimes you rescale your axis so that the units are not J/molK (example divide by DuLong Petit limit)
        returnValue = returnValue * SchottkyPerTCoef[3] ;
    EndIf
    
    return returnValue ; 
End

Function AvgEvryNpnts( wave WaveToAverage, wave DestinationWave, variable vN , variable reverseOrNo )
    //Example: running this program on {12,34,56,78,90,112} with vN=2 , reverseOrNo = 0 produces { 23, 67, 101 } [did I do my math right?]. If reverseOrNo was 1 the you get {101, 67, 23}
    int revYorN; revYorN = floor(reverseOrNo) ;//We are passing the variable from a macro and the macro cannor define ints. So we convert to int
    int N; N = floor(vN) ; //N is the number of points to average.
    
    wave wv = WaveToAverage;
    wave NewWv = DestinationWave;
    
    int PntsInWv = numpnts(wv) ; //printf "Number of points in input wave: %g.\r", PntsInWv
    
    int tester1 = mod( PntsInWv, N ) ;
    
    If (tester1 != 0)//If the number of points in the wave is not evenly divisible by (N) then I assume there is a problem
        print "Number of points in wave not divisible by %g.\r", vN ;
    Else
    
        int PntsInNewWv = PntsInWv / N ; //At the moment, NewWv does not have this many points, but we will make it have this many points
        //printf "Points Currently in new wave: %g.\r", numpnts(NewWv); //printf "Target number of points in new wave: %g.\r", PntsInNewWv ;
        Redimension/N=(PntsInNewWv) NewWv ; //printf "Checking number of points in new wave: %g.\r", numpnts(NewWv) ;
	    
        int j = 0;
        int k = 0;
        
        do //I think there is a 'shorthand' way of doing this in igor. NewWv[p] = 0 ... using the index (p) tells igor to loop through all points in the wave or something ... I would have to check
            NewWv[k] = 0;
            k += 1;
        while( k < PntsInNewWv )
        
        do
            k = 0
            do
                NewWv[k] += wv[N*k + j ] ;
                k += 1;
            while(k < PntsInNewWv)
            j+=1;
        while ( j < N )
        
        NewWv = NewWv / N ;
        
        DestinationWave = NewWv ;
        If( revYorN )
            Reverse DestinationWave ;
        EndIf
    EndIf
    
End

Function MakeDestinationWaves( variable vN )
    
    int N; N = floor( vN ) ;//'theUserInput' was originally made by a macro and macros can't define int, so we convert to int

    //You may need to change all of the names here...AND THESE names are used elsewhere [poor programming ... but use find&replace]
    Make/O/N=(N) '_Cp[J/K]' ; Make/O/N=(N) '_CpErr[J/K]' ;
    
    Make/O/N=(N) '_Cp[J/K∙mol]' ; Make/O/N=(N) '_CpErr[J/K∙mol]' ;
    
    Make/O/N=(N) '_Cp/mol∙3R' ; Make/O/N=(N) '_CpErr/mol∙3R' ;//These are indeed unitless
    
    Make/O/N=(N) '_Cp[J/K∙molF.u.]' ; Make/O/N=(N) '_CpErr[J/K∙molF.u.]' ;
    
    Make/O/N=(N) '_T[K]' ; Make/O/N=(N) '_T²[K²]' ;
     
    Make/O/N=(N) '_Cp/T[J/K²]' ; Make/O/N=(N) '_Cp/T Err[J/K²]' ;
    
    Make/O/N=(N) '_Cp/T[J/K²∙molYb]' ; Make/O/N=(N) '_Cp/T Err[J/K²∙molYb]'
    
    Make/O/N=(N) '_▲Cp[J/K∙mol]' ;  Make/O/N=(N) '_▲Cp[J/K∙molF.u.]' ;
    
    Make/O/N=(N) '_▲Cp/mol∙3R' ; Make/O/N=(N) '_▲Cp/mol∙3R Err';
    
    Make/O/N=(N) '_▲Cp/mol∙3R Fit' ; Make/O/N=(N) '_▲Cp/mol∙3R FitErr';
    
    Make/O/N=(N) '_▲Cp/T[J/K²∙molYb]' ; Make/O/N=(N) '_▲Cp/T[J/K²∙molYb] Err'; 
    
    Make/O/N=(N) '_▲Cp/T[J/K²∙molYb] Fit' ; Make/O/N=(N) '_▲Cp/T[J/K²∙molYb] FitErr';
    
    Make/O/N=(N) '_CpDebyeResidual' ; Make/O/N=200 'TscaledToFit' ;
    
    Make/O/N=(N) 'TotalCp[J/K]' ; Make/O/N=(N) 'PercntCp' ;//This is just to check and make sure that the sample has most of the heat capacity at low temperature. It has nothing to do with the Delta Cp's
    
    Make/O/N=(N) 'Scale x Rln(1+G)' ; Make/O/N=(N) 'One' = 1 ;
    
    Make/O/N=(N) '_dS Err²[J/K∙molYb]' ; Make/O/N=(N) '_S Err[J/K∙molYb]' ; //We do not need to make the wave for (S) here since that can be made during the numerical integration
    //We make (dS Err)^2 and (S Err) since the method I use is (S Err)[i] = sqrt( Sum_0^i (dS Err)^2 )
    
    Make/O/N=(N) '_dSmag Err²[J/K∙molYb]' ; Make/O/N=(N) '_Smag Err[J/K∙molYb]' ;

End

Macro SetDestinationWaves()
    variable N ; N = 'theUserInput'[4] ;//N is the number of points to average per actual data point
    variable theScale; theScale = 'theUserInput'[0]
    variable revYes = 1; //We want to reverse our waves since we start from large T
    
    Variable MolsFU ; MolsFU = 'theUserInput'[1] ;//See Macro ProcessData( theScale, molsFu, NatomsFu, NelecFu, PointsToAvg , NumDataPntsAfterAvg ) if you aren't already familiar with 'theUserInput'
    Variable MolsAtom; MolsAtom = MolsFU * 'theUserInput'[2] ; //See Macro ProcessData( theScale, molsFu, NatomsFu, NelecFu, PointsToAvg , NumDataPntsAfterAvg ) if you aren't already familiar with 'theUserInput'
    Variable NelecInFU ; NelecInFU = 'theUserInput'[3] ;
    Variable DuLongPetit = 'theUserInput'[7]
    
    //NOTE: I have not changed the name of the raw data waves so the names might suggest discrepency in units by factor of 10^n. However, the math should be correct
    //Not many comments in this section; I have tried to make the nameing convention transparent enough
    AvgEvryNpnts( 'Samp_HC___J_K_' , '_Cp[J/K]' , N, revYes ) ; AvgEvryNpnts( 'Samp_HC_Err___J_K_' , '_CpErr[J/K]' , N, revYes ) ;
    
    '_Cp[J/K]' = '_Cp[J/K]' * theScale ; '_CpErr[J/K]' = '_CpErr[J/K]' * theScale ;//Rescale Cp
    
    '_Cp[J/K∙mol]' = '_Cp[J/K]' / MolsAtom ; '_CpErr[J/K∙mol]' = '_CpErr[J/K]' / MolsAtom ;//Order is important, '_Cp[J/K] has already been scaled by theScale (which is what we want)
    
    '_Cp/mol∙3R' = '_Cp[J/K∙mol]' / DuLongPetit; '_CpErr/mol∙3R' = '_CpErr[J/K∙mol]' / DuLongPetit ;//Order is important, '_Cp[J/K∙mol]' has already been scaled by MolsAtom (which is what we want)
    
    '_Cp[J/K∙molF.U.]' = '_Cp[J/K]' / MolsFU ; '_CpErr[J/K∙molF.U.]' = '_CpErr[J/K]' / MolsFU ;
    
    AvgEvryNpnts('Sample_Temp__Kelvin_' , '_T[K]' , N, revYes ) ; AvgEvryNpnts('Temp_Squared__K_2_' , '_T²[K²]' , N , revYes );
    
    AvgEvryNpnts( 'Samp_HC_Temp___J_K_K_' , '_Cp/T[J/K²]' , N , revYes ) ; '_Cp/T[J/K²]' = '_Cp/T[J/K²]' * theScale ;
    
    '_Cp/T Err[J/K²]' = '_CpErr[J/K]' / '_T[K]' ;
    
    '_Cp/T[J/K²∙molYb]' = '_Cp/T[J/K²]' / MolsFU / NelecInFU ; '_Cp/T Err[J/K²∙molYb]' = '_Cp/T Err[J/K²]' / MolsFU / NelecInFU ;
    
    AvgEvryNpnts('Total_HC___J_K_' , 'TotalCp[J/K]' , N , revYes ) ; 'TotalCp[J/K]' = 'TotalCp[J/K]' * theScale ;
    
    'PercntCp' = 100 * '_Cp[J/K]' / ( 'TotalCp[J/K]' ) ;
    
    variable j = 0;
    '_dS Err²[J/K∙molYb]'[j] = 0; j+=1 ;
    do
        '_dS Err²[J/K∙molYb]'[j] = ( ( '_Cp/T Err[J/K²∙molYb]'[j] + '_Cp/T Err[J/K²∙molYb]'[j-1] ) * ( '_T[K]'[j] - '_T[K]'[j-1] ) / 2 )^2 ;
        j+=1 ;
        
    while( j < numpnts( '_dS Err²[J/K∙molYb]' ) )
    
    Integrate/METH=1 '_Cp/T[J/K²∙molYb]'/X='_T[K]'/D='_S[J/K∙molYb]';DelayUpdate //Recall that Cv = T dS/dT, so Integrate[Cv/T, {T,0,Temp}] = S(Temp)
    
    j = 0; 
    do
        '_S Err[J/K∙molYb]'[j] = Sqrt( Sum( '_dS Err²[J/K∙molYb]', 0, j ) ) ;
        j+=1 ;
    while( j < numpnts( '_S Err[J/K∙molYb]' ) )
     
    
//    j = 0; variable k ; string troubleshooter;
//    do
//        k = j * ( numpnts('_T[K]') -1 )/( numpnts('TscaledToFit') -1 );
//        'TscaledToFit'[j] = '_T[K]'[ j * ( numpnts('_T[K]') -1 )/( numpnts('TscaledToFit') -1 )]
//        j += 1 ;
//        sprintf troubleshooter "%g", k ;
//    while( j < numpnts( 'TscaledToFit' ) )
//    print troubleshooter;

EndMacro

Macro UpdateDeltaCp( NmbrDataPnts )

    variable NmbrDataPnts ;
    
    variable j = 0; //The fit trace produced by igor has 200 points. Here we need to interpolate it down to NmbrDataPnts
    do
        //'_▲Cp/mol∙3R'[j] = '_Cp/mol∙3R' - 'fit__Cp/mol∙3R'[ ( numpnts( 'fit__Cp/mol∙3R' ) - 1 ) * ( '_T[K]'[j] - '_T[K]'[0] ) / ( '_T[K]'['theUserInput'[5] - 1] - '_T[K]'[0] ) ] ;   // We use the fact that igro uses linear interpolation. I suggest typing this out and referencing 'Interpolation in Wave Assignments' in the Igor manual
        '_▲Cp/mol∙3R'[j] = '_Cp/mol∙3R'[j] - DebyeInterpolationWithLinearT( 'DebyeLinT_Coef' , '_T[K]'[j] ) ;
        'fit__Cp/mol∙3R+'[j] = DebyeInterpolationWithLinearT( { 'DebyeLinT_coef'[0] -'DebyeLinT_sigma'[0] , 'DebyeLinT_coef'[1] + 'DebyeLinT_sigma'[1],1/24.9435, 1/2} , '_T[K]'[j] );//Here, we use the fact that lowering the Debye temperature will raise the fit value and raising the GammaHT coefficient will raise the fit value
        'fit__Cp/mol∙3R-'[j] = DebyeInterpolationWithLinearT( { 'DebyeLinT_coef'[0] +'DebyeLinT_sigma'[0] , 'DebyeLinT_coef'[1] - 'DebyeLinT_sigma'[1],1/24.9435, 1/2} , '_T[K]'[j] );
        j+=1 ;
    while( j < NmbrDataPnts )
    '_▲Cp/T[J/K²∙molYb]' = '_▲Cp/mol∙3R' * 24.9435 / '_T[K]' * theUserInput[2] / theUserInput[3] ;
    '_▲Cp/mol∙3R Err' = sqrt( (0.68/2)^2 * ( 'fit__Cp/mol∙3R+' - 'fit__Cp/mol∙3R-' )^2 + '_CpErr/mol∙3R'^2 ) ; //The factor of 0.68 means 'one standard deviation' and the factor of 1/2 is since we say + or - err
    '_▲Cp/T[J/K²∙molYb] Err' = '_▲Cp/mol∙3R Err' * 24.9435 / '_T[K]' * theUserInput[2] / theUserInput[3] ;
    j = 0;
    '_dSmag Err²[J/K∙molYb]'[j] = 0; j+=1 ;
    do
        '_dSmag Err²[J/K∙molYb]'[j] = ( ( '_▲Cp/T[J/K²∙molYb] Err'[j] + '_▲Cp/T[J/K²∙molYb] Err'[j-1] ) * ( '_T[K]'[j] - '_T[K]'[j-1] ) / 2 )^2 ;
        j+=1 ; 
    while( j < numpnts( '_dSmag Err²[J/K∙molYb]' ) )
    Integrate/METH=1 '_▲Cp/T[J/K²∙molYb]'/X='_T[K]'/D='_Smag[J/K∙molYb]';DelayUpdate //Recall that Cv = T dS/dT, so Integrate[Cv/T, {T,0,Temp}] = S(Temp)
    j = 0; 
    do
        '_Smag Err[J/K∙molYb]'[j] = Sqrt( Sum( '_dSmag Err²[J/K∙molYb]', 0, j ) ) ;
        j+=1 ;
    while( j < numpnts( '_Smag Err[J/K∙molYb]' ) )
EndMacro

Macro DisplayData()

    variable NmbrDataPnts ;
    NmbrDataPnts = 'theUserInput'[5] ;
    
    variable TwoHundredK2 = BinarySearch( '_T²[K²]', 200 ) ; variable EightHundredK2 = BinarySearch( '_T²[K²]', 800 ) ;//This is used in the fit for Cp/T vs T^2 and also perhaps for DeltaCp vs T
    
    //Not many comments in this section. Try making plots, fitting functions, and changing display with the Igor Graphical User Interface [GUI] and use the 'to cmd' feature. That is what I did to help write this code
    // Also, when labeling axes with the [GUI], double-click the axis then find 'Axis Label' tab [and click on it] then find 'Special' [and click on it], then click 'Igor TeX'

    Display '_Cp/mol∙3R' vs '_T[K]' ; ModifyGraph mode=3, mrkThick=3.5 ;
    Label left "\Z18 \\$WMTEX$ \\frac{Cp}{mol \\cdot 3R } \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T \\left[ K  \\right] \\$/WMTEX$" ; 
    ErrorBars '_Cp/mol∙3R' Y,wave=('_CpErr/mol∙3R','_CpErr/mol∙3R');
    
    variable hundredK = BinarySearch('_T[K]', 100) ; //We are fitting a Debye interpolation integral to the Cp/T graph (Cp normalized to molFormulaUnit
    variable dataLen = numpnts('_T[K]') ;
    Make/D/N=4/O DebyeLinT_coef ;
    DebyeLinT_coef[0] = {300,0.01,1/24.9435,1/2} ;
    FuncFit/X=1/H="0011"/TBOX=768 DebyeInterpolationWithLinearT DebyeLinT_coef '_Cp/mol∙3R'[hundredK,dataLen-1] /X='_T[K]' /W='_CpErr/mol∙3R' /I=1 /D /R /A=0
    Make/O/N=2 'DebyeLinT_sigma'; 'DebyeLinT_sigma' = W_sigma; //Igor will always make the errors in coefficients go to a wave named W_sigma. So you need to put these values in the wave you actually want them to be in
    Make/O/N=(NmbrDataPnts) 'fit__Cp/mol∙3R+'; Make/O/N=(NmbrDataPnts) 'fit__Cp/mol∙3R-';
    //'fit__Cp/mol∙3R+' = DebyeInterpolationWithLinearT( { 'DebyeLinT_Coef' } , '_T[K]' )
    
    UpdateDeltaCp( NmbrDataPnts ) ;
    AppendToGraph/L 'One' vs '_T[K]' ; ModifyGraph rgb('One')=(0,65535,0) ;
    AppendToGraph/R '_▲Cp/mol∙3R' vs '_T[K]' ;
    ModifyGraph mode('_▲Cp/mol∙3R')=3,marker('_▲Cp/mol∙3R')=19 ;
    ErrorBars '_▲Cp/mol∙3R' Y,wave=('_▲Cp/mol∙3R Err','_▲Cp/mol∙3R Err')
    
    variable Grat = 'theUserInput'[8] ;
    Make/D/N=2/O Schottky_coef
    Schottky_coef[0] = {90,Grat,1,1/24.9435}
    FuncFit/X=1/H="0101"/TBOX=768 Schottky Schottky_coef '_▲Cp/mol∙3R'[EightHundredK2, dataLen-1] /X='_T[K]' /W='_▲Cp/mol∙3R Err' /I=1 /D /R /A=0
    Make/O/N=1 'Schottky_sigma' ; 'Schottky_sigma' = W_sigma ;
    //Make/O/N=200 'test'; 'test' = 'fit__▲Cp/mol∙3R' * 24.9435 / 'TscaledToFit' * theUserInput[2] / theUserInput[3] ;//I have had difficulty making the TscaledToFit
    
    variable j = 0 ;
    do
        '_▲Cp/mol∙3R Fit'[j] = Schottky( Schottky_coef, '_T[K]'[j] ) ;
        '_▲Cp/T[J/K²∙molYb] Fit'[j] = '_▲Cp/mol∙3R Fit'[j] * 24.9435 / '_T[K]'[j] * theUserInput[2] / theUserInput[3]
        j += 1 ;
    while( j < numpnts( '_▲Cp/mol∙3R Fit' ) )
    
    variable SRln1pG = Schottky_coef[2] * 'theUserInput'[2] / 'theUserInput'[3] * 8.3144626 * ln( 1 + 1/Grat ) ;//This has units of S/molYb
    'Scale x Rln(1+G)' = SRln1pG ;
    
    Label right "\Z18 \\$WMTEX$  \\frac{\\Delta Cp}{3R \\cdot mol} \\$/WMTEX$" ;
    
    Display '_▲Cp/T[J/K²∙molYb]' vs '_T[K]' ; ModifyGraph mode=3, mrkThick = 3.5 ;
//    Make/D/N=2/O SchottkyPerT_coef
//    SchottkyPerT_coef[0] = {90,Grat,1, 1 }
//    FuncFit/X=1/H="0101"/TBOX=768 SchottkyPerT SchottkyPerT_coef '_▲Cp/T[J/K²∙molYb]'[TwoHundredK2, dataLen-1] /X='_T[K]' /I=1 /D /R /A=0 ;
//    Make/O/N=1 'SchottkyPerT_sigma' ; 'SchottkyPerT_sigma' = W_sigma ;
    AppendToGraph '_▲Cp/T[J/K²∙molYb] Fit' vs '_T[K]' ; //ModifyGraph rgb('_▲Cp/T[J/K²∙molYb] Fit')=(25535,0,65535) ;
    AppendToGraph/R 'Scale x Rln(1+G)' vs '_T[K]' ; ModifyGraph rgb('Scale x Rln(1+G)')=(0,65535,0) ;
    AppendToGraph/R '_Smag[J/K∙molYb]' vs '_T[K]' ;
    ModifyGraph mode('_Smag[J/K∙molYb]')=3,rgb('_Smag[J/K∙molYb]')=(1,16019,65535),marker('_Smag[J/K∙molYb]')=19 ;
    ErrorBars '_▲Cp/T[J/K²∙molYb]' Y,wave=('_▲Cp/T[J/K²∙molYb] Err','_▲Cp/T[J/K²∙molYb] Err')
    ErrorBars '_Smag[J/K∙molYb]' Y,wave=('_Smag Err[J/K∙molYb]','_Smag Err[J/K∙molYb]');
    ModifyGraph log(bottom)=1 ;
    
    Label right "\Z18 \\$WMTEX$ S^{mag} \\left[ \\frac{J}{K \\cdot molYb} \\right] \\$/WMTEX$" ;
    Label left "\Z18 \\$WMTEX$ \\Delta Cp/T \\left[ \\frac{J}{\\cdot K² \\cdot molYb} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T \\left[ K  \\right] \\$/WMTEX$" ;
    
    Display '_Cp[J/K∙mol]' vs '_T²[K²]' ; ModifyGraph mode=3 ; ModifyGraph mrkThick=3.5 ;
    Label left "\Z18 \\$WMTEX$ Cp \\left[ \\frac{J}{3R \\cdot K \\cdot mol} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T² \\left[ K²  \\right] \\$/WMTEX$" ; 
    
    Display '_Cp/T[J/K²∙molYb]' vs '_T²[K²]' ; ModifyGraph mode=3 ; ModifyGraph mrkThick=3.5 ;
    ErrorBars '_Cp/T[J/K²∙molYb]' Y,wave=('_Cp/T Err[J/K²∙molYb]','_Cp/T Err[J/K²∙molYb]') ;
    Make/D/O/N=2 LowTLin_coef
    CurveFit/X=1/TBOX=768 line kwCWave=LowTLin_coef, '_Cp/T[J/K²∙molYb]'[TwoHundredK2, EightHundredK2] /X='_T²[K²]' /W='_Cp/T Err[J/K²∙molYb]' /I=1 /D /R /A=0 
    Make/O/N=2 LowLinT_sigma ; LowLinT_sigma = W_sigma ;
    Label left "\Z18 \\$WMTEX$ \\frac{Cp}{T} \\left[ \\frac{J}{K² \\cdot molYb} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T² \\left[ K²  \\right] \\$/WMTEX$" ; 
    
    Display '_Cp/T[J/K²∙molYb]' vs '_T[K]' ; ModifyGraph mode=3 ; ModifyGraph mrkThick=3.5 ;
    Label left "\Z18 \\$WMTEX$ \\frac{Cp}{T} \\left[ \\frac{J}{K² \\cdot molYb} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T \\left[ K  \\right] \\$/WMTEX$" ;
    
    Display 'PercntCp' vs '_T[K]' ; ModifyGraph mode=3 ; ModifyGraph mrkThick=3.5 ;
    Label left "\Z18 \\$WMTEX$ \%Cp \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$  T \\left[ °K  \\right] \\$/WMTEX$"
    
    Display '_S[J/K∙molYb]' vs '_T[K]' ; ModifyGraph mode=3 ; ModifyGraph mrkThick=3.5 ;
    Label left "\Z18 \\$WMTEX$ S \\left[ \\frac{J}{K \\cdot molYb} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$  T \\left[ K  \\right] \\$/WMTEX$"
    
    
    
    TileWindows/O=1/C/P ;

End

Macro ProcessData( theScale, molsFu, NatomsFu, NelecFu, PointsToAvg , NumDataPntsAfterAvg, DivBy3R_orNo, Grat )
    
    variable theScale ; //So if you get data in units of [uJ] and want to display in units of [J], enter 1e-6 for theScale
    variable molsFu ; //The number of mols of formula unit in your sample. For example, the molecular weight of formula unit for Yb2Si2Al is 429 [g/mol]. molsFu = massSample/[molecular wieght of formula unit]
    variable NatomsFu ; //How many atoms are in the formula unit. Example, there are 5 atoms in the formula unit for Yb2Si2Al
    variable NelecFu ; //How many ??conduction?? electrons are carried in the formula unit. Example, NelecFu for Yb2Si2Al is 2 since there are 2Yb atoms in the unit cell and each Yb corresponds to one ??conduction?? electron
    variable PointsToAvg ; //Sometimes your machine measures multiple data points per temperature value so you can average them and have 'better' data
    variable NumDataPntsAfterAvg ; //After you averagel the raw data points, how many data points are you left with? Example, if your raw data file has 180 pints, but you are averaging 3 points per (T) value, NumDataPntsAfterAvg = 60
    variable DivBy3R_orNo ;
    variable Grat ; //This will be set to a global variable for the relative degeneracy used to fit the SchottyPeak
    
    //BIG COMMENT. Sometimes you scale by 1/(3R) = 1/(DuLong-Petit constant). This version does so.
    
    If(theScale !=0)//It is common for me to want to re-process the data, but not change the user inputs. If you leave theScale = 0 then the user inputs stay as is. There will certainly be a bug if you don't create the user inputs and leave theScale=0
        Make/O/N=9 'theUserInput' = { theScale, molsFu, NatomsFu, NelecFu, PointsToAvg , NumDataPntsAfterAvg , DivBy3R_orNo , 24.94339, Grat }//24.94339 is 3*R, 3*gas constant, DuLong-Petit constant. If DivBy3R_orNo=1 then you divide Cp by 3R
    EndIf //This is my way of making these into 'global' variables in the experiment, which can be viewed by viewing the wave 'theUserInput' in the data browser
    
    theScale = 'theUserInput'[0] ; molsFu = 'theUserInput'[1] ; NatomsFu = 'theUserInput'[2] ; //This is awkward and I will edit later. I think all I need is to set NumDataPntsAfterAvg = 'theUserInput'[5]
    NelecFu= 'theUserInput'[3] ; PointsToAvg = 'theUserInput'[4] ;
    NumDataPntsAfterAvg = 'theUserInput'[5] ; DivBy3R_orNo = 'theUserInput'[6] ;
    
    KillAllGraphs() ;
    
    MakeDestinationWaves( NumDataPntsAfterAvg ) ;//When we are naming the waves, we just need to know the number of data points which will end up in them
    SetDestinationWaves() ;//When we set the values of the waves, we need to know the scale and whether to divide by 3R
    DisplayData() ; DoUpdate;
    
EndMacro

Macro MakePlotFolders()
    cd root:
    variable j = 0 ; string cmd;
    do
        sprintf cmd "NewDataFolder/O \'%s\' ;" , 'PlotFolders'[j] ; Execute cmd ;
        j += 1 ;
    while( j < numpnts( PlotFolders ) )
EndMacro

Macro GetAllDataToRoot()
    cd root:
    variable j; string cmd;
    j=0;  
    do
        cd root:'Cp vs T' ;
        sprintf cmd "Make/O/N=60 \'%s Cp\' ;" , root:SpecimenFolders[j]; Execute cmd;
        sprintf cmd "\'%s Cp\' = root:%s:\'_Cp/mol∙3R\'" , root:SpecimenFolders[j] , root:SpecimenFolders[j]; Execute cmd;
        j += 1
    while( j < numpnts( root:SpecimenFolders ) )
    j=0
    do
        cd root:'Cp/T vs T²' ;
        sprintf cmd "Make/O/N=60 \'%s Cp/T\' ;", root:SpecimenFolders[j]; Execute cmd;
        sprintf cmd "\'%s Cp/T\' = root:%s:\'_Cp/T[J/K²∙molYb]\'", root:SpecimenFolders[j], root:SpecimenFolders[j]; Execute cmd;
        j += 1 ;
    while( j < numpnts( rood:SpecimenFolders ) )
    
EndMacro

Macro DisplayAllData()
    
    
    
    variable j; // j will be used as an index
    string cmd;
    cd root:
    Display/N='CpVsT'
    j = 0;
    do
        cd $('SpecimenFolders'[j]) ;
        
        AppendToGraph/C=(2e4*j,2e4*j,0) '_Cp/mol∙3R' vs '_T[K]' ;
        sprintf cmd "ModifyGraph mode('_Cp/mol∙3R'#%g)=3,marker('_Cp/mol∙3R'#%g)=(18+j) ;",j,j ; Execute cmd ;
        AppendToGraph/C=(2e4*j,2e4*j,0) 'fit__Cp/mol∙3R' ;
        AppendToGraph/R/C=(2e4*j,2e4*j,0) '_▲Cp/mol∙3R' vs '_T[K]' ;
        sprintf cmd "ModifyGraph mode('_▲Cp/mol∙3R'#%g)=3,marker('_▲Cp/mol∙3R'#%g)=(18+j) ;",j,j ; Execute cmd ;
        AppendToGraph/R/C=(2e4*j,2e4*j,0) 'fit__▲Cp/mol∙3R' ;
        Label left "\Z18 \\$WMTEX$ \\frac{Cp}{mol \\cdot 3R } \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T \\left[ K  \\right] \\$/WMTEX$" ; 
        cd root: ;//We have to go back to the root: directory to see the 'Folders' wave
        j+=1 ;    
    While( j < numpnts( SpecimenFolders ) )
    
    cd root:
    Display/N='CpDivTvsT2'
    j = 0;
    do
        cd $('SpecimenFolders'[j]) ;
        
        AppendToGraph/C=(2e4*j,2e4*j,0) '_Cp/T[J/K²∙molYb]' vs '_T²[K²]' ;
        sprintf cmd "ModifyGraph mode('_Cp/T[J/K²∙molYb]'#%g)=3,marker('_Cp/T[J/K²∙molYb]'#%g)=(18+j) ;",j,j ; Execute cmd ;
        AppendToGraph/C=(2e4*j,2e4*j,0) 'fit__Cp/T[J/K²∙molYb]' ;
        Label left "\Z18 \\$WMTEX$ \\frac{Cp}{T} \\left[ \\frac{J}{K² \\cdot molYb} \\right] \\$/WMTEX$" ; Label bottom "\Z18 \\$WMTEX$ T² \\left[ K²  \\right] \\$/WMTEX$" ;
        cd root: ;//We have to go back to the root: directory to see the 'Folders' wave
        j+=1 ;    
    While( j < numpnts( SpecimentFolders ) )
    
EndMacro



// display/N='CpVsT'
// cd $"root:UKy46"
// AppendToGraph root:UKy46:'Samp_HC___J_K_' vs root:UKy46:'Sample_Temp__Kelvin_'
// AppendToGraph $"root:UKy46:'Samp_HC___J_K_'" vs $"root:UKy46:'Sample_Temp__Kelvin_'"
//
// ModifyGraph marker('fit__Cp/mol∙3R')=19,msize('fit__Cp/mol∙3R')=2.2,mrkThick('fit__Cp/mol∙3R')=2.2,rgb('fit__Cp/mol∙3R')=(1,16019,65535)
//
//